# coding=utf-8
from traceback import format_exc
from pyPdf import PdfFileReader
import datetime, os
from StringIO import StringIO
import cmislibalf		# automatico
from cmislib.model import CmisClient
from configuracoes import FILE_LOG, FILE_PDF, FILE_ERROR, CMIS_ROOT, CMIS_SENHA, CMIS_URL, CMIS_USUARIO, CMIS_ARQUIVO, CMIS_ARQUIVO_SCRIPT_NOME, DEBUG
from script import checaScript, criarScript, criaEstrutura, checaEstrutura


with open(FILE_LOG, u'w') as logFile, open(FILE_ERROR, u'w') as errorFile:
	def log(texto):
		l = u'\n%s - %s'%(datetime.datetime.now().strftime(u'[%H:%M:%S.%f - %d/%m/%Y]'), texto)
		logFile.write(l.encode(u'utf8'))

		if DEBUG:
			print l,

	try:
		log(u'Abrindo conexao: %s'%CMIS_URL )

		client = CmisClient( CMIS_URL, CMIS_USUARIO, CMIS_SENHA)
		REPO = client.getDefaultRepository()

		log(u'Cmis aberto: %s'%CMIS_URL)

		#----------------------------------------------------
		# verifica se o arquivo CONTADOR-PDF existe
		# 	se nao existir, CRIA:
		#		-	o diretorio
		#		-	o arquivo em branco para receber as URLs
		#		-	o script
		#----------------------------------------------------
		log(u"---------------------------------------------")
		log(u"Checando estrutura necessaria...")
		if not checaEstrutura(REPO, CMIS_ROOT + CMIS_ARQUIVO):
			log(u"Criando Estrutura [%s]..."%(CMIS_ROOT + CMIS_ARQUIVO))
			criaEstrutura(REPO, CMIS_ROOT + CMIS_ARQUIVO)
		log(u"Estrutura OK!")
		log(u"---------------------------------------------")
		log(u"Checando Script para RULES")
		if not checaScript(REPO, CMIS_ARQUIVO_SCRIPT_NOME):
			log(u"Criando Script [%s]..."%(CMIS_ARQUIVO_SCRIPT_NOME))
			criarScript(REPO, CMIS_ARQUIVO_SCRIPT_NOME)
		log(u"Script OK!")
		log(u"---------------------------------------------")
		log(u"Lendo o arquivo: %s"%( CMIS_ROOT + CMIS_ARQUIVO ))
		#-------------------------------------------------
		#	pega o arquivo com os nodeRef's
		#-------------------------------------------------
		nodesRefs = REPO.getObjectByPath(CMIS_ROOT + CMIS_ARQUIVO)
		refsFile = nodesRefs.getContentStream()

		log(u'Apagando conteudo...')
		nodesRefs.deleteContentStream()
		nodesRefs.setContentStream(StringIO(u'\n'))
		log(u'Gerando lista...')
		listaRefs = refsFile.readlines()
		refsFile.close()

		log(u'Criando Copia...')
		copia = REPO.getObjectByPath(CMIS_ROOT).createDocumentFromString(
			u'copia - %s.txt'%( unicode(datetime.datetime.now()).replace(u':', u'-') ),
			contentString = u''.join(listaRefs),
		 	contentType = 'text/plain',
			contentEncoding =u'utf-8'
		)
		log(u'Copia criada: %s'%( copia.name ))

		log(u'Retirando conteudo em branco...')
		listaRefs = [x.strip() for x in listaRefs if x.strip()]

		totalError = 0
		log(u'Buscando arquivos no repositorio...')
		for nodeRef in listaRefs:
			try:
				node = REPO.getObject(nodeRef)
				log(u'Criando arquivo')
				with open(FILE_PDF, u'wb') as arquivo:
					diretorio = node.getPaths()[0].replace(u'\n', u'')

					node.reload()
					log(u'Baixando arquivo: %s'%( diretorio ))
					_baixado = node.getContentStream()
					arquivo.write( _baixado.read() )
					_baixado.close()
					log(u'Arquivo baixado')

				with open(FILE_PDF, u'rb') as arquivo:
					log(u'Abrindo o arquivo como PDF')
					pdf = PdfFileReader( arquivo )
					log(u'Contando paginas')
					pagina = pdf.getNumPages()
					del pdf
					log(u'Paginas encontradas: %s'%pagina)

				os.remove(FILE_PDF)

				log(u'Atualizando propriedades...')
				node.updateProperties({u'ftmg:numeroPaginas': pagina})

				log(u'Propriedades atualizadas\n')

			except IOError as e:
				totalError += 1
				errorFile.write( unicode(u'Erro ao abrir o arquivo "%s", verifique se ele existe'%(e.filename) ).encode(u'utf8') )
				log(u'Erro ao abrir o arquivo "%s", verifique se ele existe'%(e.filename) )

			except Exception as e:
				log(u'Erro: %s'%(e.message))
				log(format_exc())

		log(u'Buscando concluida')
		errorFile.write( unicode(u'Total de arquivos com erro: "%s"'%(totalError)).encode(u'utf8') )

	except:
		log(u'---------------------------------------------')
		log(u'Erro geral: ')
		log(format_exc())

	else:
		#	apaga a copia de seguranca
		copia.delete()

if not DEBUG:
	with open(FILE_LOG, u'r') as logFile, open(FILE_ERROR, u'r') as errorFile:
		client = CmisClient( CMIS_URL, CMIS_USUARIO, CMIS_SENHA)
		REPO = client.getDefaultRepository()
		ROOT = REPO.getObjectByPath(CMIS_ROOT)

		def salvarRelatorio(arquivo, nome):

			_diretorio = criaEstrutura(REPO, CMIS_ROOT + u'logs/' + nome, False)
			_diretorio.createDocument( nome, contentFile=arquivo, contentType=u"text/plain", contentEncoding=u'utf-8')

			if DEBUG:
				print u'\nArquivo salvo em: %s'%(CMIS_ROOT + u'logs/')

		salvarRelatorio( logFile, logFile.name[logFile.name.rfind(u'/')+1:] )
		salvarRelatorio( errorFile, errorFile.name[errorFile.name.rfind(u'/')+1:] )

os.remove(FILE_LOG)
os.remove(FILE_ERROR)